#ifdef DOXYGEN_SHOULD_SKIP_THIS

include $(shell rospack find mk)/cmake.mk

###############################################################################
# Help
###############################################################################

help:
	@echo "install       : installs files like a typical ./configure, make, make install."
	@echo "install/strip : as above, but strips libraries and binaries as well."
	@echo "uninstall     : only if ecl_build_utilities' ecl_uninstall macro is called."
	@echo "examples      : for any binaries using the ecl_add_example macro."
  	@echo "benchmarks    : for any binaries using the ecl_add_benchmarks macro."
  	@echo "utilities     : for any binaries using the ecl_add_utilities macro."
  	@echo "apps          : rolls examples/benchmarks/utilities targets into one."
  	@echo "sources       : like install, but only gathers sources, not libs and headers."
  	@echo "package       : only if cpack information has been built in to your CMakeLists.txt"

###############################################################################
# Install targets
###############################################################################

install:
	@if [ -d build ]; then \
	    cd build; \
	    sudo $(MAKE) install; \
	else \
		$(MAKE) all; \
		cd build; \
		sudo $(MAKE) install; \
	fi 

install/strip:
	@if [ -d build ]; then \
	    cd build; \
	    sudo $(MAKE) install/strip; \
	else \
		$(MAKE) all; \
		cd build; \
		sudo $(MAKE) install/strip; \
	fi 

uninstall:
	@if [ -d build ]; then \
	    cd build; \
	    sudo $(MAKE) uninstall; \
	else \
	    echo No build directory; \
	fi

sources:
	@if [ -d build ]; then \
	    cd build; \
	    sudo $(MAKE) sources; \
	else \
		mkdir build; \
		cd build; \
		cmake ..; \
		sudo $(MAKE) sources; \
	fi 

###############################################################################
# Options (aka ./configure and co.)
###############################################################################

configure:
	@if [ -d build ]; then \
		make clean; \
		mkdir -p build; \
	    cd build; \
	    cmake ..; \
	else \
		mkdir -p build; \
	    cd build; \
	    cmake ..; \
	fi 

options:
	@if [ -d build ]; then \
	    cd build; \
	    ccmake .; \
	else \
		mkdir -p build; \
		cd build; \
		cmake ..; \
		ccmake .; \
	fi 

###############################################################################
# Apps
###############################################################################

apps:
	@if [ -d build ]; then \
	    cd build; \
	    $(MAKE) apps; \
	else \
		$(MAKE) all; \
		cd build; \
		$(MAKE) apps; \
	fi


examples:
	@if [ -d build ]; then \
	    cd build; \
	    $(MAKE) examples; \
	else \
		$(MAKE) all; \
		cd build; \
		$(MAKE) examples; \
	fi

benchmarks:
	@if [ -d build ]; then \
	    cd build; \
	    $(MAKE) benchmarks; \
	else \
		$(MAKE) all; \
		cd build; \
		$(MAKE) benchmarks; \
	fi

utilities:
	@if [ -d build ]; then \
	    cd build; \
	    $(MAKE) utilities; \
	else \
		$(MAKE) all; \
		cd build; \
		$(MAKE) utilities; \
	fi


###############################################################################
# Packages
###############################################################################
	
package:
	@if [ -d build ]; then \
	    cd build; \
	    $(MAKE) package; \
	else \
		$(MAKE) all; \
		cd build; \
		$(MAKE) package; \
	fi

upload: package
	cd build; \
	$(MAKE) upload

#endif DOXYGEN_SHOULD_SKIP_THIS
	