import roslib; 
roslib.load_manifest('roscreate')
roslib.load_manifest('ecl_create')

# This sets up a typical ecl package.

import os
import sys
import shutil
import roslib.packages
from roscreate.core import author_name
from ecl_create.core import read_template
from ecl_create.core import instantiate_template
from ecl_create.core import short_package_name

def get_templates(package):
    templates = {}
    templates['mainpage.dox'] = read_template(os.path.join(roslib.packages.get_pkg_dir('ecl_create'),'templates','default','mainpage.dox'))
    templates['Makefile'] = read_template(os.path.join(roslib.packages.get_pkg_dir('ecl_create'),'templates','default','Makefile'))
    templates['CMakeLists.txt'] = read_template(os.path.join(roslib.packages.get_pkg_dir('ecl_create'),'templates','default','CMakeLists.txt'))
    templates['manifest.xml'] = read_template(os.path.join(roslib.packages.get_pkg_dir('ecl_create'),'templates','default','manifest.xml'))
    templates[os.path.join('src','CMakeLists.txt')] = read_template(os.path.join(roslib.packages.get_pkg_dir('ecl_create'),'templates','default','src','CMakeLists.txt'))
    templates[os.path.join('src','lib','CMakeLists.txt')] = read_template(os.path.join(roslib.packages.get_pkg_dir('ecl_create'),'templates','default','src','lib','CMakeLists.txt'))
    templates[os.path.join('src','lib','foo.cpp')] = read_template(os.path.join(roslib.packages.get_pkg_dir('ecl_create'),'templates','default','src','lib','foo.cpp'))
    templates[os.path.join('src','test','CMakeLists.txt')] = read_template(os.path.join(roslib.packages.get_pkg_dir('ecl_create'),'templates','default','src','test','CMakeLists.txt'))
    templates[os.path.join('src','test','first_test.cpp')] = read_template(os.path.join(roslib.packages.get_pkg_dir('ecl_create'),'templates','default','src','test','first_test.cpp'))
    templates[os.path.join('include','CMakeLists.txt')] = read_template(os.path.join(roslib.packages.get_pkg_dir('ecl_create'),'templates','default','include','CMakeLists.txt'))
    templates[os.path.join('include','ecl','CMakeLists.txt')] = read_template(os.path.join(roslib.packages.get_pkg_dir('ecl_create'),'templates','default','include','ecl','CMakeLists.txt'))
    templates[os.path.join('include','ecl',short_package_name(package)+'.hpp')] = read_template(os.path.join(roslib.packages.get_pkg_dir('ecl_create'),'templates','default','include','ecl','short_package_name.hpp'))
    templates[os.path.join('include','ecl',short_package_name(package),'CMakeLists.txt')] = read_template(os.path.join(roslib.packages.get_pkg_dir('ecl_create'),'templates','default','include','ecl','short_package_name','CMakeLists.txt'))
    templates[os.path.join('include','ecl',short_package_name(package),'foo.hpp')] = read_template(os.path.join(roslib.packages.get_pkg_dir('ecl_create'),'templates','default','include','ecl','short_package_name','foo.hpp'))
    return templates

def create_package(package, depends):
    
    p = os.path.abspath(package)
    os.makedirs(os.path.join(p,"src","lib"))
    os.makedirs(os.path.join(p,"src","test"))
    os.makedirs(os.path.join(p,"include","ecl",short_package_name(package)))
    print "Created ecl package directories."

    templates = get_templates(package)
    for filename, template in templates.iteritems():
        contents = instantiate_template(template, package, short_package_name(package), package, author_name(), depends)
        try:
            p = os.path.abspath(os.path.join(package, filename))
            f = open(p, 'w')
            f.write(contents)
            print "Created package file", p
        finally:
            f.close()
    

def main():
    from optparse import OptionParser
    parser = OptionParser(usage="usage: %prog <package-name> [dependencies...]")
    options, args = parser.parse_args()
    if not args:
        parser.error("you must specify a package name and optionally also list package dependencies")
    package = args[0]

    # validate dependencies and turn into XML
    depends = args[1:]

    for d in depends:
        try:
            roslib.packages.get_pkg_dir(d)
        except roslib.packages.InvalidROSPkgException:
            print >> sys.stderr, "ERROR: dependency [%s] cannot be found"%d
            sys.exit(1)
    depends = ''.join(['  <depend package="%s"/>\n'%d for d in depends])

    create_package(package, depends )
    print "\nPlease edit %s/manifest.xml and mainpage.dox to finish creating your package"%package

if __name__ == "__main__":
    main()
    
